@@ -1,3 +1,7 @@
+!---------------------------------------------------------------
+! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+!---------------------------------------------------------------
+
 module Derivation  ! ʬ黻׻⥸塼
 
 contains
@@ -77,7 +81,11 @@ subroutine div( x, y, u, v, val, undeff, hx, hy )
      do j=1,ny
         do i=1,nx
            if(u(i,j)/=undeff.or.v(i,j)/=undeff)then
-              val(i,j)=(dudx(i,j)+dvdy(i,j))/(scalex(i,j)*scaley(i,j))
+              if(scalex(i,j)/=0.0.and.scaley(i,j)/=0.0)then
+                 val(i,j)=(dudx(i,j)+dvdy(i,j))/(scalex(i,j)*scaley(i,j))
+              else
+                 val(i,j)=0.0
+              end if
            else
               val(i,j)=undeff
            end if
@@ -102,7 +110,11 @@ subroutine div( x, y, u, v, val, undeff, hx, hy )
 
      do j=1,ny
         do i=1,nx
-           val(i,j)=(dudx(i,j)+dvdy(i,j))/(scalex(i,j)*scaley(i,j))
+           if(scalex(i,j)/=0.0.and.scaley(i,j)/=0.0)then
+              val(i,j)=(dudx(i,j)+dvdy(i,j))/(scalex(i,j)*scaley(i,j))
+           else
+              val(i,j)=0.0
+           end if
         end do
      end do
   end if
@@ -194,7 +206,11 @@ subroutine curl( x, y, u, v, val, undeff, hx, hy, ord )
      do j=1,ny
         do i=1,nx
            if(u(i,j)/=undeff.or.v(i,j)/=undeff)then
-              val(i,j)=(dvdx(i,j)-dudy(i,j))/(scalex(i,j)*scaley(i,j))
+              if(scalex(i,j)/=0.0.and.scaley(i,j)/=0.0)then
+                 val(i,j)=(dvdx(i,j)-dudy(i,j))/(scalex(i,j)*scaley(i,j))
+              else
+                 val(i,j)=0.0
+              end if
            else
               val(i,j)=undeff
            end if
@@ -219,7 +235,11 @@ subroutine curl( x, y, u, v, val, undeff, hx, hy, ord )
 
      do j=1,ny
         do i=1,nx
-           val(i,j)=(dvdx(i,j)-dudy(i,j))/(scalex(i,j)*scaley(i,j))
+           if(scalex(i,j)/=0.0.and.scaley(i,j)/=0.0)then
+              val(i,j)=(dvdx(i,j)-dudy(i,j))/(scalex(i,j)*scaley(i,j))
+           else
+              val(i,j)=0.0
+           end if
         end do
      end do
   end if
@@ -345,8 +365,12 @@ subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
         do j=1,ny
            do i=1,nx
               if(u(i,j,k)/=undeff.or.v(i,j,k)/=undeff.or.w(i,j,k)/=undeff)then
-                 val(i,j,k)=(dudx(i,j,k)+dvdy(i,j,k)+dwdz(i,j,k))/  &
-  &                         (scalex(i,j,k)*scaley(i,j,k)*scalez(i,j,k))
+                 if(scalex(i,j,k)/=0.0.and.scaley(i,j,k)/=0.0.and.scalez(i,j,k)/=0.0)then
+                    val(i,j,k)=(dudx(i,j,k)+dvdy(i,j,k)+dwdz(i,j,k))/  &
+  &                            (scalex(i,j,k)*scaley(i,j,k)*scalez(i,j,k))
+                 else
+                    val(i,j,k)=0.0
+                 end if
               else
                  val(i,j,k)=undeff
               end if
@@ -385,8 +409,12 @@ subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
      do k=1,nz
         do j=1,ny
            do i=1,nx
-              val(i,j,k)=(dudx(i,j,k)+dvdy(i,j,k)+dwdz(i,j,k))/  &
-  &                      (scalex(i,j,k)*scaley(i,j,k)*scalez(i,j,k))
+              if(scalex(i,j,k)/=0.0.and.scaley(i,j,k)/=0.0.and.scalez(i,j,k)/=0.0)then
+                 val(i,j,k)=(dudx(i,j,k)+dvdy(i,j,k)+dwdz(i,j,k))/  &
+  &                         (scalex(i,j,k)*scaley(i,j,k)*scalez(i,j,k))
+              else
+                 val(i,j,k)=0.0
+              end if
            end do
         end do
      end do
@@ -566,27 +594,51 @@ subroutine grad_1d( x, u, val, undef, hx )
         if(u(i+1)==undef.or.u(i-1)==undef)then
            val(i)=undef
         else
-           val(i)=(u(i+1)-u(i-1))/(scalex(i)*(x(i+1)-x(i-1)))
+           if(scalex(i)/=0.0)then
+              val(i)=(u(i+1)-u(i-1))/(scalex(i)*(x(i+1)-x(i-1)))
+           else
+              val(i)=0.0
+           end if
         end if
      end do
 !-- ǡΤʤξüν ---
      if(u(1)==undef.or.u(2)==undef)then
         val(1)=undef
      else
-        val(1)=(u(2)-u(1))/(scalex(1)*(x(2)-x(1)))
+        if(scalex(1)/=0.0)then
+           val(1)=(u(2)-u(1))/(scalex(1)*(x(2)-x(1)))
+        else
+           val(1)=0.0
+        end if
      end if
      if(u(nx)==undef.or.u(nx-1)==undef)then
         val(nx)=undef
      else
-        val(nx)=(u(nx)-u(nx-1))/(scalex(nx)*(x(nx)-x(nx-1)))
+        if(scalex(nx)/=0.0)then
+           val(nx)=(u(nx)-u(nx-1))/(scalex(nx)*(x(nx)-x(nx-1)))
+        else
+           val(nx)=0.0
+        end if
      end if
   else
      do i=2,nx-1
-        val(i)=(u(i+1)-u(i-1))/(scalex(i)*(x(i+1)-x(i-1)))
+        if(scalex(i)/=0.0)then
+           val(i)=(u(i+1)-u(i-1))/(scalex(i)*(x(i+1)-x(i-1)))
+        else
+           val(i)=0.0
+        end if
      end do
 !-- ǡΤʤξüν ---
-     val(1)=(u(2)-u(1))/(scalex(1)*(x(2)-x(1)))
-     val(nx)=(u(nx)-u(nx-1))/(scalex(nx)*(x(nx)-x(nx-1)))
+     if(scalex(1)/=0.0)then
+        val(1)=(u(2)-u(1))/(scalex(1)*(x(2)-x(1)))
+     else
+        val(1)=0.0
+     end if
+     if(scalex(nx)/=0.0)then
+        val(nx)=(u(nx)-u(nx-1))/(scalex(nx)*(x(nx)-x(nx-1)))
+     else
+        val(nx)=0.0
+     end if
   end if
 
 end subroutine grad_1d
@@ -648,8 +700,16 @@ subroutine grad_2d( x, y, u, valx, valy, undeff, hx, hy )
      do j=1,ny
         do i=1,nx
            if(u(i,j)/=undeff)then
-              valx(i,j)=valx(i,j)/scalex(i,j)
-              valy(i,j)=valy(i,j)/scaley(i,j)
+              if(scalex(i,j)/=0.0)then
+                 valx(i,j)=valx(i,j)/scalex(i,j)
+              else
+                 valx(i,j)=0.0
+              end if
+              if(scaley(i,j)/=0.0)then
+                 valy(i,j)=valy(i,j)/scaley(i,j)
+              else
+                 valy(i,j)=0.0
+              end if
 !-- , else ʤΤ, grad_1d 롼ǤǤ undeff äƤ뤫ƱȤ
 !-- ʤΤ, 䰦.
            end if
@@ -668,8 +728,16 @@ subroutine grad_2d( x, y, u, valx, valy, undeff, hx, hy )
 
      do j=1,ny
         do i=1,nx
-           valx(i,j)=valx(i,j)/scalex(i,j)
-           valy(i,j)=valy(i,j)/scaley(i,j)
+           if(scalex(i,j)/=0.0)then
+              valx(i,j)=valx(i,j)/scalex(i,j)
+           else
+              valx(i,j)=0.0
+           end if
+           if(scaley(i,j)/=0.0)then
+              valy(i,j)=valy(i,j)/scaley(i,j)
+           else
+              valy(i,j)=0.0
+           end if
         end do
      end do
 
@@ -757,9 +825,21 @@ subroutine grad_3d( x, y, z, u, valx, valy, valz, undeff, hx, hy, hz )
         do j=1,ny
            do i=1,nx
               if(u(i,j,k)/=undeff)then
-                 valx(i,j,k)=valx(i,j,k)/scalex(i,j,k)
-                 valy(i,j,k)=valy(i,j,k)/scaley(i,j,k)
-                 valz(i,j,k)=valz(i,j,k)/scalez(i,j,k)
+                 if(scalex(i,j,k)/=0.0)then
+                    valx(i,j,k)=valx(i,j,k)/scalex(i,j,k)
+                 else
+                    valx(i,j,k)=0.0
+                 end if
+                 if(scaley(i,j,k)/=0.0)then
+                    valy(i,j,k)=valy(i,j,k)/scaley(i,j,k)
+                 else
+                    valy(i,j,k)=0.0
+                 end if
+                 if(scalez(i,j,k)/=0.0)then
+                    valz(i,j,k)=valz(i,j,k)/scalez(i,j,k)
+                 else
+                    valz(i,j,k)=0.0
+                 end if
               end if
            end do
         end do
@@ -788,9 +868,21 @@ subroutine grad_3d( x, y, z, u, valx, valy, valz, undeff, hx, hy, hz )
      do k=1,nz
         do j=1,ny
            do i=1,nx
-              valx(i,j,k)=valx(i,j,k)/scalex(i,j,k)
-              valy(i,j,k)=valy(i,j,k)/scaley(i,j,k)
-              valz(i,j,k)=valz(i,j,k)/scalez(i,j,k)
+              if(scalex(i,j,k)/=0.0)then
+                 valx(i,j,k)=valx(i,j,k)/scalex(i,j,k)
+              else
+                 valx(i,j,k)=0.0
+              end if
+              if(scaley(i,j,k)/=0.0)then
+                 valy(i,j,k)=valy(i,j,k)/scaley(i,j,k)
+              else
+                 valy(i,j,k)=0.0
+              end if
+              if(scalez(i,j,k)/=0.0)then
+                 valz(i,j,k)=valz(i,j,k)/scalez(i,j,k)
+              else
+                 valz(i,j,k)=0.0
+              end if
            end do
         end do
      end do
@@ -802,6 +894,123 @@ end subroutine grad_3d
 !-----------------------------------------
 !-----------------------------------------
 
+subroutine zast_2_vel_2d( x, y, zast, u, v, w, uh, vh, wh, undef )
+! terrain following ɸϤɾƤ®ʿȱľѴ.
+! , 3 ǥȷϤѴΤǤϤʤ, terrain following ɸϤ
+! ®ʬǥȷѴ.
+! , ʿˤϥǥȷϤˤΤбƤ.
+! ⤷, ɸѴ, vert_coord_conv ⥸塼ѤΤ.
+  implicit none
+  real, dimension(:), intent(in) :: x  ! κɸ [m]
+  real, dimension(:), intent(in) :: y  ! κɸ [m]
+  real, dimension(size(x),size(y)), intent(in) :: zast  ! terrain Ϥ,  1  [m]
+  real, dimension(size(x),size(y)), intent(in) :: u  ! zast ˤ [m/s]
+  real, dimension(size(x),size(y)), intent(in) :: v  ! zast ˤ [m/s]
+  real, dimension(size(x),size(y)), intent(in) :: w  ! zast ˤľ [m/s]
+  real, dimension(size(x),size(y)), intent(inout) :: uh  ! ǥȷϤˤ [m/s]
+  real, dimension(size(x),size(y)), intent(inout) :: vh  ! ǥȷϤˤ [m/s]
+  real, dimension(size(x),size(y)), intent(inout) :: wh  ! ǥȷϤˤľ [m/s]
+  real, intent(in), optional :: undef  ! »
+  integer :: i, j, nx, ny
+  real, dimension(size(x),size(y)) :: dx, dy
+  real :: cosx, sinx, cosy, siny
+
+  nx=size(x)
+  ny=size(y)
+
+  call grad_2d( x, y, zast, dx, dy )
+
+  if(present(undef))then
+     do j=1,ny
+        do i=1,nx
+           if(u(i,j)/=undef.and.v(i,j)/=undef.and.w(i,j)/=undef)then
+              cosx=1.0/sqrt(1.0+dx(i,j)*dx(i,j))
+              cosy=1.0/sqrt(1.0+dy(i,j)*dy(i,j))
+              sinx=dx(i,j)*cosx
+              siny=dy(i,j)*cosy
+              uh(i,j)=u(i,j)*cosx
+              vh(i,j)=v(i,j)*cosy
+              wh(i,j)=w(i,j)+u(i,j)*sinx+v(i,j)*siny
+           else
+              uh(i,j)=undef
+              vh(i,j)=undef
+              wh(i,j)=undef
+           end if
+        end do
+     end do
+  else
+     do j=1,ny
+        do i=1,nx
+           cosx=1.0/sqrt(1.0+dx(i,j)*dx(i,j))
+           cosy=1.0/sqrt(1.0+dy(i,j)*dy(i,j))
+           sinx=dx(i,j)*cosx
+           siny=dy(i,j)*cosy
+           uh(i,j)=u(i,j)*cosx
+           vh(i,j)=v(i,j)*cosy
+           wh(i,j)=w(i,j)+u(i,j)*sinx+v(i,j)*siny
+        end do
+     end do
+  end if
+
+end subroutine zast_2_vel_2d
+
+!-----------------------------------------
+!-----------------------------------------
+
+subroutine zast_2_vel_3d( x, y, zast, u, v, w, uh, vh, wh, undef )
+! terrain following ɸϤɾƤ®ʿȱľѴ.
+! , 3 ǥȷϤѴΤǤϤʤ, terrain following ɸϤ
+! ®ʬǥȷѴ.
+! , ʿˤϥǥȷϤˤΤбƤ.
+! ⤷, ɸѴ, vert_coord_conv ⥸塼ѤΤ.
+  implicit none
+  real, dimension(:,:,:), intent(in) :: zast  ! terrain Ϥιٺɸ [m]
+  real, dimension(size(zast,1)), intent(in) :: x  ! κɸ [m]
+  real, dimension(size(zast,2)), intent(in) :: y  ! κɸ [m]
+  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(in) :: u  ! zast ˤ [m/s]
+  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(in) :: v  ! zast ˤ [m/s]
+  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(in) :: w  ! zast ˤľ [m/s]
+  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(inout) :: uh  ! ǥȷϤˤ [m/s]
+  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(inout) :: vh  ! ǥȷϤˤ [m/s]
+  real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(inout) :: wh  ! ǥȷϤˤľ [m/s]
+  real, intent(in), optional :: undef  ! »
+  integer :: i, j, k, nx, ny, nz
+  real, dimension(size(zast,1),size(zast,2),size(zast,3)) :: dx, dy
+  real :: cost, sint
+
+  nx=size(zast,1)
+  ny=size(zast,2)
+  nz=size(zast,3)
+
+  if(present(undef))then
+
+!$omp parallel default(shared)
+!$omp do private(k)
+     do k=1,nz
+        call zast_2_vel_2d( x, y, zast(:,:,k), u(:,:,k), v(:,:,k), w(:,:,k),  &
+  &                         uh(:,:,k), vh(:,:,k), wh(:,:,k), undef )
+     end do
+!$omp end do
+!$omp end parallel
+
+  else
+
+!$omp parallel default(shared)
+!$omp do private(k)
+     do k=1,nz
+        call zast_2_vel_2d( x, y, zast(:,:,k), u(:,:,k), v(:,:,k), w(:,:,k),  &
+  &                         uh(:,:,k), vh(:,:,k), wh(:,:,k) )
+     end do
+!$omp end do
+!$omp end parallel
+
+  end if
+
+end subroutine zast_2_vel_3d
+
+!-----------------------------------------
+!-----------------------------------------
+
 subroutine laplacian_1d( x, u, val, undef )
 ! 1 Υ顼ѿΥץ饷׻
 ! $\frac{\partial ^2p}{\partial x^2} $ 
