@@ -1,12 +1,18 @@
+!---------------------------------------------------------------
+! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+!---------------------------------------------------------------
+
 module ffts
 ! fft ϢΥ֥롼
+  use Math_Const
 
 contains
 
-subroutine ffttp_1d( nx, a, b, csign, prim, prim_fact )
+subroutine ffttp_1d( nx, a, b, csign, prim, prim_fact, omega_fix, omegan_fix )
 !  Temperton's FFT
 !
 !  1d  fft  csign Ȥ, Ѵ, Ѵ롼
+  use Math_Const
   implicit none
   integer, intent(in) :: nx  ! ǿ
   complex, intent(in), dimension(0:nx-1) :: a  ! 
@@ -18,21 +24,23 @@ subroutine ffttp_1d( nx, a, b, csign, prim, prim_fact )
   ! , 'x' ΤȤǤ, prim_fact ꤵƤ, Τ٤ FFT .
   integer, dimension(5), optional :: prim_fact  ! prim = x ΤȤꤹ
   ! Τ٤ FFT Ԥ. prim_fact=(/a,b,c,d,e/) : 2^a*3^b*5^c*7^d*e
+  complex, dimension(0:nx-1,0:nx-1), intent(in), optional :: omega_fix
+                       ! ;ǰбž
+  complex, dimension(0:nx-1,0:nx-1), intent(in), optional :: omegan_fix  ! ž
   integer, allocatable, dimension(:) :: l, m, n  ! κ
-  real, parameter :: pi=3.14159265
-  complex, parameter :: img=(0.0,1.0)
-  complex :: fact, ctmp
+  complex :: ctmp
   integer :: stat, counter, base
   integer :: i, j, k, id, jd, kd  ! do loop 
   integer, parameter, dimension(4) :: prim_dim=(/2, 3, 5, 7/)  ! ǰ
   integer, dimension(4) :: prim_num  ! ǰΤ٤
-  complex, dimension(0:1,0:1) :: omega2
-  complex, dimension(0:2,0:2) :: omega3
-  complex, dimension(0:4,0:4) :: omega5
-  complex, dimension(0:6,0:6) :: omega7
   complex, allocatable, dimension(:,:) :: omega
   complex, dimension(0:nx-1,0:nx-1) :: omegan
   complex, dimension(0:nx-1) :: c  ! tmp array
+  complex, dimension(0:1,0:1) :: omega2  ! ȿ 2 βž
+  complex, dimension(0:2,0:2) :: omega3  ! ȿ 3 βž
+  complex, dimension(0:4,0:4) :: omega5  ! ȿ 5 βž
+  complex, dimension(0:6,0:6) :: omega7  ! ȿ 7 βž
+
 
   base=nx
   prim_num=0
@@ -42,6 +50,12 @@ subroutine ffttp_1d( nx, a, b, csign, prim, prim_fact )
      b(i)=a(i)
   end do
 
+  if(real(romega2(1,1))/=-1.0.or.aimag(romega2(1,1))/=1.0.or.  &
+  &  real(iomega2(1,1))/=-1.0.or.aimag(iomega2(1,1))/=1.0)then
+     ! math_const  rotate_array 롼󤬷׻Ƥʤ, ư.
+     call rotate_array()
+  end if
+
 !-- ǰʬ򤹤
   if(present(prim))then
      if(prim=='o')then
@@ -56,146 +70,113 @@ subroutine ffttp_1d( nx, a, b, csign, prim, prim_fact )
            end do
            base=prim_fact(5)
         else
+           call prim_calc( nx, prim_num, base )
            do i=1,4
-              do while(mod(base,prim_dim(i))==0)
-                 base=base/prim_dim(i)
-                 prim_num(i)=prim_num(i)+1
-                 counter=counter+1
-              end do
+              counter=counter+prim_num(i)
            end do
-        end if  
+        end if
  
         if(base==1)then
            counter=counter-1
         end if
-   
+ 
         if(counter/=0)then  ! prim=='o' ǤäƤ, ǰʬǤʤ DFT .
            allocate(l(counter+1))
            allocate(m(counter+1))
            allocate(n(counter+1))
            stat=0
+           l=0
+           m=0
+           n=0
    
            do i=1,4
               if(prim_num(i)/=0)then
                  select case(prim_dim(i))
                  case(2)
                     n(stat+1:stat+prim_num(i))=2
-                    stat=stat+prim_num(i)
-   
                  case(3)
                     n(stat+1:stat+prim_num(i))=3
-                    stat=stat+prim_num(i)
-   
                  case(5)
                     n(stat+1:stat+prim_num(i))=5
-                    stat=stat+prim_num(i)
-   
                  case(7)
                     n(stat+1:stat+prim_num(i))=7
-                    stat=stat+prim_num(i)
-   
                  end select
+                 stat=stat+prim_num(i)
               end if
            end do
            if(base/=1)then
               n(counter+1)=base
            end if
-write(*,*) n
         end if
      end if
   end if
 
 !-- ž
 
-  select case(csign)
-  case('r')
-     fact=cos(2.0*pi/real(nx))-img*sin(2.0*pi/real(nx))
-     if(counter/=0)then
-        omega2=1.0
-        omega2(1,1)=-1.0
+  if(counter/=0)then
+     select case (csign)
+     case ('r')
+        do j=0,1
+           do i=0,1
+              omega2(i,j)=romega2(i,j)
+           end do
+        end do
         do j=0,2
            do i=0,2
-              omega3(i,j)=cos(2.0*pi*i*j/3.0)-img*sin(2.0*pi*i*j/3.0)
+              omega3(i,j)=romega3(i,j)
            end do
         end do
         do j=0,4
            do i=0,4
-              omega5(i,j)=cos(2.0*pi*i*j/5.0)-img*sin(2.0*pi*i*j/5.0)
+              omega5(i,j)=romega5(i,j)
            end do
         end do
         do j=0,6
            do i=0,6
-              omega7(i,j)=cos(2.0*pi*i*j/7.0)-img*sin(2.0*pi*i*j/7.0)
+              omega7(i,j)=romega7(i,j)
            end do
         end do
-
-        if(base/=1)then
-           allocate(omega(0:base-1,0:base-1))
-           do j=0,base-1
-              do i=0,base-1
-                 omega(i,j)=cos(2.0*pi*i*j/real(base))-img*sin(2.0*pi*i*j/real(base))
-              end do
-           end do
-        end if
-     else
-        allocate(omega(0:nx-1,0:nx-1))
-        do j=0,nx-1
-           do i=0,nx-1
-              omega(i,j)=cos(2.0*pi*i*j/real(nx))-img*sin(2.0*pi*i*j/real(nx))
+     case ('i')
+        do j=0,1
+           do i=0,1
+              omega2(i,j)=iomega2(i,j)
            end do
         end do
-     end if
-
-     do j=0,nx-1
-        do i=0,nx-1
-           omegan(i,j)=cos(2.0*pi*i*j/real(nx))-img*sin(2.0*pi*i*j/real(nx))
-        end do
-     end do
-
-
-  case('i')
-     if(counter/=0)then
-        fact=exp(2.0*img*pi/real(nx))
-        fact=cos(2.0*pi/real(nx))+img*sin(2.0*pi/real(nx))
-        omega2=1.0
-        omega2(1,1)=-1.0
         do j=0,2
            do i=0,2
-              omega3(i,j)=cos(2.0*pi*i*j/3.0)+img*sin(2.0*pi*i*j/3.0)
+              omega3(i,j)=iomega3(i,j)
            end do
         end do
         do j=0,4
            do i=0,4
-              omega5(i,j)=cos(2.0*pi*i*j/5.0)+img*sin(2.0*pi*i*j/5.0)
+              omega5(i,j)=iomega5(i,j)
            end do
         end do
         do j=0,6
            do i=0,6
-              omega7(i,j)=cos(2.0*pi*i*j/7.0)+img*sin(2.0*pi*i*j/7.0)
+              omega7(i,j)=iomega7(i,j)
            end do
         end do
+     end select
+  end if
 
-        if(base/=1)then
-           allocate(omega(0:base-1,0:base-1))
-           do j=0,base-1
-              do i=0,base-1
-                 omega(i,j)=cos(2.0*pi*i*j/real(base))+img*sin(2.0*pi*i*j/real(base))
-              end do
-           end do
-        end if
-     end if
-
-     do j=0,nx-1
-        do i=0,nx-1
-           omegan(i,j)=cos(2.0*pi*i*j/real(nx))+img*sin(2.0*pi*i*j/real(nx))
+  if(present(omegan_fix))then
+     allocate(omega(0:base-1,0:base-1))
+     do j=0,base-1
+        do i=0,base-1
+           omega(i,j)=omega_fix(i,j)
         end do
      end do
-
-  case default
-     write(*,*) "******** ERROR : csign is bad. **********"
-     write(*,*) "Stop!"
-     stop
-  end select
+     do j=0,nx-1
+        do i=0,nx-1
+           omegan(i,j)=omegan_fix(i,j)
+        end do
+     end do
+  else
+     allocate(omega(0:base-1,0:base-1))
+     call rotate_calc( nx, csign, (/prim_num(1), prim_num(2), prim_num(3),  &
+  &                    prim_num(4), base/), omega, omegan )
+  end if
 
 !-- FFT 
 
@@ -264,10 +245,12 @@ end subroutine
 
 !-----------------------------------------------
 
-subroutine ffttp_2d( nx, ny, a, b, csign, prim, prim_fact )
+subroutine ffttp_2d( nx, ny, a, b, csign, prim, prim_fact, omega_fix,  &
+  &                  omegan_fix )
 !  Temperton's FFT (2d ver)
 !
 !  2d  fft  csign Ȥ, Ѵ, Ѵ롼
+  use Math_Const
   implicit none
   integer, intent(in) :: nx  ! ǿ 1
   integer, intent(in) :: ny  ! ǿ 2
@@ -281,13 +264,23 @@ subroutine ffttp_2d( nx, ny, a, b, csign, prim, prim_fact )
   integer, dimension(5), optional :: prim_fact  ! prim = x ΤȤꤹ
   ! Τ٤ FFT Ԥ. prim_fact=(/a,b,c,d/) : 2^a*3^b*5^c*7^d
   ! , prim_fact ꤹ٤ nx Τ٤Ǥ뤳Ȥ.
+  complex, dimension(0:nx-1,0:nx-1), intent(in), optional :: omega_fix  ! ;ǰбž
+  complex, dimension(0:nx-1,0:nx-1), intent(in), optional :: omegan_fix  ! ž
   integer :: i
 
   if(present(prim))then
      if(present(prim_fact))then
-        do i=0,ny-1
-           call ffttp_1d( nx, a(0:nx-1,i), b(0:nx-1,i), csign, prim, prim_fact )
-        end do
+        if(present(omegan_fix))then
+           do i=0,ny-1
+              call ffttp_1d( nx, a(0:nx-1,i), b(0:nx-1,i), csign, prim,  &
+  &                          prim_fact, omega_fix(0:nx-1,0:nx-1),  &
+  &                          omegan_fix(0:nx-1,0:nx-1) )
+           end do
+        else
+           do i=0,ny-1
+              call ffttp_1d( nx, a(0:nx-1,i), b(0:nx-1,i), csign, prim, prim_fact )
+           end do
+        end if
      else
         do i=0,ny-1
            call ffttp_1d( nx, a(0:nx-1,i), b(0:nx-1,i), csign, prim )
@@ -303,10 +296,11 @@ end subroutine
 
 !-----------------------------------------------
 
-subroutine r2c_ffttp_1d( nx, a, b, prim, prim_fact )
+subroutine r2c_ffttp_1d( nx, a, b, prim, prim_fact, omega_fix, omegan_fix )
 ! 1  FFT ׻롼
 !
 ! ϼ¿ǹԤ, ʣǿ֤.
+  use Math_Const
   implicit none
   integer, intent(in) :: nx  ! 1 ǡǿ
   real, intent(in), dimension(0:nx-1) :: a  ! ϼ¿ǡ
@@ -322,11 +316,11 @@ subroutine r2c_ffttp_1d( nx, a, b, prim, prim_fact )
   ! , 'x' ΤȤǤ, prim_fact ꤵƤ, Τ٤ FFT .
   integer, dimension(5), intent(in), optional :: prim_fact  ! prim = x ΤȤꤹ
   ! Τ٤ FFT Ԥ. prim_fact=(/a,b,c,d/) : 2^a*3^b*5^c*7^d
-  ! Ϥ٤ nx/2 ΤȤΤ٤Ǥ.
+  ! Ϥ٤ nx Τ٤Ǥ.
+  complex, dimension(0:nx/2-1,0:nx/2-1), intent(in), optional :: omega_fix  ! ;ǰбž
+  complex, dimension(0:nx/2-1,0:nx/2-1), intent(in), optional :: omegan_fix  ! ž
   complex, dimension(0:nx/2-1) :: c, d
-  complex, parameter :: img=(0.0,1.0)
-  real, parameter :: pi=3.14159265
-  integer :: i, j, k
+  integer :: i
   integer, dimension(5) :: prim_num
 
   if(mod(nx,2)/=0)then
@@ -347,7 +341,13 @@ subroutine r2c_ffttp_1d( nx, a, b, prim, prim_fact )
         prim_num=prim_fact
         prim_num(1)=prim_num(1)-1  ! ǡǿȾʬˤʤäƤ뤿,
                                    ! 2 Τ٤ 1 ĸäƤ.
-        call ffttp_1d( nx/2, c, d, 'r', prim, prim_num )
+        if(present(omegan_fix))then
+           call ffttp_1d( nx/2, c, d, 'r', prim, prim_num,  &
+  &                       omega_fix(0:nx/2-1,0:nx/2-1),  &
+  &                       omegan_fix(0:nx/2-1,0:nx/2-1) )
+        else
+           call ffttp_1d( nx/2, c, d, 'r', prim, prim_num )
+        end if
      else
         call ffttp_1d( nx/2, c, d, 'r', prim )
      end if
@@ -372,10 +372,11 @@ end subroutine
 
 !-----------------------------------------------
 
-subroutine c2r_ffttp_1d( nx, a, b, prim, prim_fact )
+subroutine c2r_ffttp_1d( nx, a, b, prim, prim_fact, omega_fix, omegan_fix )
 ! 1  FFT Ѵ׻롼
 !
 ! ʣǿǹԤ,¿֤.
+  use Math_Const
   implicit none
   integer, intent(in) :: nx  ! 1 ǡǿ
   complex, intent(in), dimension(0:nx/2-1) :: a  ! ʣǿǡ
@@ -391,10 +392,10 @@ subroutine c2r_ffttp_1d( nx, a, b, prim, prim_fact )
   ! , 'x' ΤȤǤ, prim_fact ꤵƤ, Τ٤ FFT .
   integer, intent(in), optional, dimension(5) :: prim_fact  ! prim = x ΤȤꤹ
   ! Τ٤ FFT Ԥ. prim_fact=(/a,b,c,d/) : 2^a*3^b*5^c*7^d
+  complex, dimension(0:nx/2-1,0:nx/2-1), intent(in), optional :: omega_fix  ! ;ǰбž
+  complex, dimension(0:nx/2-1,0:nx/2-1), intent(in), optional :: omegan_fix  ! ž
   complex, dimension(0:nx/2-1) :: c, d
-  complex, parameter :: img=(0.0,1.0)
-  real, parameter :: pi=3.14159265
-  integer :: i, j, k
+  integer :: i
   integer, dimension(5) :: prim_num
 
   if(mod(nx,2)/=0)then
@@ -416,9 +417,15 @@ subroutine c2r_ffttp_1d( nx, a, b, prim, prim_fact )
   if(present(prim))then
      if(present(prim_fact))then
         prim_num=prim_fact
-        prim_num(1)=prim_num(1)-1  ! ǡǿȾʬˤʤäƤ뤿,
+        prim_num(1)=prim_num(1)-1    ! ǡǿȾʬˤʤäƤ뤿,
                                      ! 2 Τ٤ 1 ĸäƤ.
-        call ffttp_1d( nx/2, c, d, 'i', prim, prim_num )
+        if(present(omegan_fix))then
+           call ffttp_1d( nx/2, c, d, 'i', prim, prim_num,  &
+  &                       omega_fix(0:nx/2-1,0:nx/2-1),  &
+  &                       omegan_fix(0:nx/2-1,0:nx/2-1) )
+        else
+           call ffttp_1d( nx/2, c, d, 'i', prim, prim_num )
+        end if
      else
         call ffttp_1d( nx/2, c, d, 'i', prim )
      end if
@@ -436,4 +443,111 @@ end subroutine
 
 !-----------------------------------------------
 
+subroutine rotate_calc( nx, csign, prim_fact, omega, omegan )
+!  FFT ˻Ѥž׻.
+  use Math_Const
+  implicit none
+  integer, intent(in) :: nx  ! ǿ
+  character(1), intent(in) :: csign  ! ѴȽ [r=Ѵ, i=Ѵ]
+  integer, dimension(5) :: prim_fact  ! prim = x ΤȤꤹ
+  ! Τ٤ FFT Ԥ. prim_fact=(/a,b,c,d,e/) : 2^a*3^b*5^c*7^d*e
+  complex, dimension(0:prim_fact(5)-1,0:prim_fact(5)-1), intent(inout) :: omega
+  ! ;бž
+  complex, dimension(0:nx-1,0:nx-1), intent(inout) :: omegan ! ž
+  integer :: counter, base
+  integer :: i, j  ! do loop 
+
+  base=prim_fact(5)
+
+  counter=0
+  do i=1,4
+     counter=counter+prim_fact(i)
+  end do
+
+!-- ž
+
+  select case(csign)
+  case('r')
+     if(counter/=0)then
+        if(base/=1)then
+           do j=0,base-1
+              do i=0,base-1
+                 omega(i,j)=cos(2.0*pi*i*j/real(base))-img*sin(2.0*pi*i*j/real(base))
+              end do
+           end do
+        end if
+        do j=0,nx-1
+           do i=0,nx-1
+              omegan(i,j)=cos(2.0*pi*i*j/real(nx))-img*sin(2.0*pi*i*j/real(nx))
+           end do
+        end do
+
+     else
+        do j=0,nx-1
+           do i=0,nx-1
+              omega(i,j)=cos(2.0*pi*i*j/real(nx))-img*sin(2.0*pi*i*j/real(nx))
+              omegan(i,j)=omega(i,j)
+           end do
+        end do
+     end if
+
+
+  case('i')
+     if(counter/=0)then
+        if(base/=1)then
+           do j=0,base-1
+              do i=0,base-1
+                 omega(i,j)=cos(2.0*pi*i*j/real(base))+img*sin(2.0*pi*i*j/real(base))
+              end do
+           end do
+        end if
+        do j=0,nx-1
+           do i=0,nx-1
+              omegan(i,j)=cos(2.0*pi*i*j/real(nx))+img*sin(2.0*pi*i*j/real(nx))
+           end do
+        end do
+     else
+        do j=0,nx-1
+           do i=0,nx-1
+              omega(i,j)=cos(2.0*pi*i*j/real(nx))+img*sin(2.0*pi*i*j/real(nx))
+              omegan(i,j)=omega(i,j)
+           end do
+        end do
+     end if
+
+  case default
+     write(*,*) "******** ERROR : csign is bad. **********"
+     write(*,*) "Stop!"
+     stop
+  end select
+
+end subroutine
+
+!-----------------------------------------------
+
+subroutine prim_calc( n, factor, resid )
+  ! 2,3,5,7 ˤĤƤǰʬԤ.
+  implicit none
+  integer, intent(in) :: n  ! ʬ򤹤
+  integer, intent(inout), dimension(4) :: factor  ! ʬ򤷤Ȥγǰ
+  ! ΤȤ, factor(1:4)=(/a,b,c,d/) Ǥ, factor(5)=e Ȥ,
+  ! n=2^a*3^b*5^c*7^d*e ȤˤʤäƤ.
+  integer, intent(inout) :: resid  ! residual prim factor
+  integer :: i, base
+  integer, dimension(4) :: prim_dim
+
+  prim_dim=(/2,3,5,7/)
+  base=n
+  factor=(/0,0,0,0/)
+  do i=1,4
+     do while(mod(base,prim_dim(i))==0)
+        base=base/prim_dim(i)
+        factor(i)=factor(i)+1
+     end do
+  end do
+
+  resid=base
+
+end subroutine
+
 end module
