@@ -1,3 +1,7 @@
+!---------------------------------------------------------------
+! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+!---------------------------------------------------------------
+
 module file_operate  ! CReSS η׻̽ϥեǡѥ⥸塼
 
 interface read_file_gtool3_header
@@ -30,6 +34,31 @@ end subroutine read_file
 !-----------------------------------------------
 !-----------------------------------------------
 
+subroutine read_file_3d( file_name, nx, ny, nz, rec_num, var )
+  ! read_file 3 .
+  ! ܥ롼Ǥ, 쥯ȥɤ߽Ф, 1 ѿΥХȿ 4 Х
+  ! Ȳꤷɤ߽ФΤ, 4 ХȰʳΥեɤ߽ФȤ,
+  ! Ϥ꥽եν񤭴ɬפȤʤ׽
+  implicit none
+  integer, intent(in) :: nx  ! ǡ x θĿ
+  integer, intent(in) :: ny  ! ǡ y θĿ
+  integer, intent(in) :: nz  ! ǡ z θĿ
+  integer, intent(in) :: rec_num  ! ɤ߽Ф򳫻ϤǡΥ쥳ֹ
+  character(*), intent(in) :: file_name  ! ɤ߽Фǡե̾
+  real, intent(inout) :: var(nx,ny,nz)  ! ɤ߽Фǡ
+  integer :: i, j, k  ! 
+
+  open(unit=11, file=file_name, access='direct', recl=4*nx*ny, status='old')
+     do k=1,nz
+        read(11,rec=rec_num+k-1) ((var(i,j,k),i=1,nx),j=1,ny)
+     end do
+  close(unit=11, status='keep')
+
+end subroutine read_file_3d
+
+!-----------------------------------------------
+!-----------------------------------------------
+
 subroutine write_file( file_name, nx, ny, rec_num, var, mode )  ! Ͻϥ롼
   ! ܥ롼Ǥ, 쥯ȥɤ߽Ф, 1 ѿΥХȿ 4 Х
   ! Ȳꤷɤ߽ФΤ, 4 ХȰʳΥեɤ߽ФȤ,
@@ -61,6 +90,41 @@ end subroutine write_file
 !-----------------------------------------------
 !-----------------------------------------------
 
+subroutine write_file_3d( file_name, nx, ny, nz, rec_num, var, mode )
+  ! write_file 3 .
+  ! ܥ롼Ǥ, 쥯ȥɤ߽Ф, 1 ѿΥХȿ 4 Х
+  ! Ȳꤷɤ߽ФΤ, 4 ХȰʳΥեɤ߽ФȤ,
+  ! Ϥ꥽եν񤭴ɬפȤʤ׽
+  implicit none
+  integer, intent(in) :: nx  ! ǡ x θĿ
+  integer, intent(in) :: ny  ! ǡ y θĿ
+  integer, intent(in) :: nz  ! ǡ z θĿ
+  integer, intent(in) :: rec_num  ! ɤ߽Ф򳫻ϤǡΥ쥳ֹ
+  character(*), intent(in) :: file_name  ! ɤ߽Фǡե̾
+  real, intent(inout) :: var(nx,ny,nz)  ! ɤ߽Фǡ
+  character(*), optional, intent(in) :: mode  ! եν񤭽Фץ
+  integer :: i, j, k  ! 
+  character(10) :: cmode
+
+  cmode=''
+
+  if(present(mode))then
+     cmode=mode
+  else
+     cmode='unknown'
+  end if
+
+  open(unit=11, file=file_name, access='direct', recl=4*nx*ny, status=cmode)
+     do k=1,nz
+        write(11,rec=rec_num+k-1) ((var(i,j,k),i=1,nx),j=1,ny)
+     end do
+  close(unit=11, status='keep')
+
+end subroutine write_file_3d
+
+!-----------------------------------------------
+!-----------------------------------------------
+
 !subroutine read_file_valbyte( file_name, nx, ny, rec_num, byte, var )
 !  ! ǤեХȿΥǡɤ߽Ф롼
 !  implicit none
@@ -88,15 +152,17 @@ subroutine request_dim_grads( fname, dim_len )
   integer, intent(inout), dimension(4) :: dim_len  ! x,y,z,t ǿ
   character(1000), allocatable, dimension(:) :: ctl_cval
   character(100), allocatable, dimension(:,:) :: ctl_hval
-  integer, dimension(4) :: cont_flag
+  integer, dimension(5) :: cont_flag
   integer :: nl, i
   character(1000) :: tmpc
+  logical :: pdeflag
 
   nl=line_number_counter( trim(fname) )
 
   allocate(ctl_cval(nl))
   allocate(ctl_hval(10,nl))
 
+  pdeflag=.false.
   ctl_cval=''
   ctl_hval=''
   tmpc=''
@@ -118,19 +184,30 @@ subroutine request_dim_grads( fname, dim_len )
      if(tmpc(1:4)=='tdef')then
         cont_flag(4)=i
      end if
+     if(tmpc(1:4)=='pdef')then
+        cont_flag(5)=i
+        pdeflag=.true.
+     end if
   end do
 
-  read(ctl_hval(2,cont_flag(1)),*) dim_len(1)
-  read(ctl_hval(2,cont_flag(2)),*) dim_len(2)
-  read(ctl_hval(2,cont_flag(3)),*) dim_len(3)
-  read(ctl_hval(2,cont_flag(4)),*) dim_len(4)
+  if(pdeflag.eqv..true.)then
+     read(ctl_hval(2,cont_flag(1)),*) dim_len(1)
+     read(ctl_hval(2,cont_flag(2)),*) dim_len(2)
+     read(ctl_hval(2,cont_flag(3)),*) dim_len(3)
+     read(ctl_hval(2,cont_flag(4)),*) dim_len(4)
+  else
+     read(ctl_hval(2,cont_flag(5)),*) dim_len(1)
+     read(ctl_hval(3,cont_flag(5)),*) dim_len(2)
+     read(ctl_hval(2,cont_flag(3)),*) dim_len(3)
+     read(ctl_hval(2,cont_flag(4)),*) dim_len(4)
+  end if
 
 end subroutine
 
 !-----------------------------------------------
 !-----------------------------------------------
 
-subroutine request_axis_grads( fname, axis_name, x )
+subroutine request_axis_grads( fname, axis_name, x, y )
 ! GrADS Υȥե뤫 1 μǡ.
 ! , linear, levels פΤб.
   implicit none
@@ -138,20 +215,31 @@ subroutine request_axis_grads( fname, axis_name, x )
   character(*), intent(in) :: axis_name  ! ɤ߽Ф̾.
                               ! xdef, ydef, zdef  3 ѥ
   real, intent(inout) :: x(:)  ! κɸ
+  real, optional, intent(inout) :: y(:)  ! pdef ξ y κɸ
   character(1000), allocatable, dimension(:) :: ctl_cval
   character(100), allocatable, dimension(:) :: ctl_vval
   character(100), allocatable, dimension(:,:) :: ctl_hval
-  integer :: nl, i, j, k, nx, err
+  integer :: nl, i, j, nx, ny
   real :: vmin, dv
   character(1000) :: tmpc
   character(100), allocatable, dimension(:) :: dummy
 
   nl=line_number_counter( trim(fname) )
   nx=size(x)
+  if(present(y))then
+     if(trim(axis_name)=='pdef')then
+        ny=size(y)
+     else
+        write(*,*) "ERROR : request_axis_grads."
+        write(*,*) "If axis_name is pdef, argument y must need."
+        write(*,*) "STOP."
+        stop
+     end if
+  end if
 
   allocate(ctl_cval(nl))
   allocate(ctl_hval(2,nl))
-  allocate(ctl_vval(5))
+  allocate(ctl_vval(13))
   allocate(dummy(nx+3))
 
   ctl_cval=''
@@ -166,23 +254,33 @@ subroutine request_axis_grads( fname, axis_name, x )
   do i=1,nl
      tmpc=trim(adjustl(ctl_cval(i)))
      if(tmpc(1:4)==trim(axis_name))then
-        call read_file_text( trim(fname), 5, 1, ctl_vval(1:5), skip=i-1 )
-        if(trim(ctl_vval(3))=='linear')then
-           read(ctl_vval(4),*) vmin
-           read(ctl_vval(5),*) dv
-           x=(/((vmin+dv*(i-1)),i=1,nx)/)
-        else
-           if(trim(ctl_vval(3))=='levels')then
-              call read_file_text( trim(fname), nx+3, 1, dummy, skip=i-1 )
-              do j=4,nx+3
-                 read(dummy(j),*) x(j-3)
-              end do
+        if(tmpc(1:4)/='pdef')then
+           call read_file_text( trim(fname), 5, 1, ctl_vval(1:5), skip=i-1 )
+           if(trim(ctl_vval(3))=='linear')then
+              read(ctl_vval(4),*) vmin
+              read(ctl_vval(5),*) dv
+              x=(/((vmin+dv*(i-1)),i=1,nx)/)
            else
-              write(*,*) "ERROR (request_axis_grads) : ", trim(ctl_vval(3))
-              write(*,*) "Mode is neither 'levels' nor 'linear'."
-              write(*,*) "STOP."
-              stop
+              if(trim(ctl_vval(3))=='levels')then
+                 call read_file_text( trim(fname), nx+3, 1, dummy, skip=i-1 )
+                 do j=4,nx+3
+                    read(dummy(j),*) x(j-3)
+                 end do
+              else
+                 write(*,*) "ERROR (request_axis_grads) : ", trim(ctl_vval(3))
+                 write(*,*) "Mode is neither 'levels' nor 'linear'."
+                 write(*,*) "STOP."
+                 stop
+              end if
            end if
+        else
+           call read_file_text( trim(fname), 13, 1, ctl_vval(1:13), skip=i-1 )
+           read(ctl_vval(7),*) vmin
+           read(ctl_vval(12),*) dv
+           x=(/((vmin+dv*(i-1)),i=1,nx)/)
+           read(ctl_vval(8),*) vmin
+           read(ctl_vval(13),*) dv
+           y=(/((vmin+dv*(i-1)),i=1,ny)/)
         end if
      end if
   end do
@@ -255,6 +353,7 @@ subroutine read_file_grads( fname, val_name, dim_len, intvz, intvt, val )
   character(1000) :: tmpc, ctmp, header, footer, iname
   integer, dimension(10) :: cont_flag
   integer :: t_fact, year, month, day, hour, counter_start, read_flag, counter
+  integer :: year_r, month_r, day_r, hour_r
   logical, dimension(4) :: t_fact_flag, date_flag
 
   nl=line_number_counter( trim(fname) )
@@ -424,6 +523,11 @@ subroutine read_file_grads( fname, val_name, dim_len, intvz, intvt, val )
      month=12
   end select
 
+  year_r=year
+  month_r=month
+  day_r=day
+  hour_r=hour
+
 !-- reading data
 
   do i=intvt(1),intvt(2)
@@ -435,19 +539,19 @@ subroutine read_file_grads( fname, val_name, dim_len, intvz, intvt, val )
         select case (j)
         case (1)
            if(t_fact_flag(1).eqv..true.)then
-              year=year+t_fact*(i-1)
+              year=year_r+t_fact*(i-1)
            end if
         case (2)
            if(t_fact_flag(2).eqv..true.)then
-              month=month+t_fact*(i-1)
+              month=month_r+t_fact*(i-1)
            end if
         case (3)
            if(t_fact_flag(3).eqv..true.)then
-              day=day+t_fact*(i-1)
+              day=day_r+t_fact*(i-1)
            end if
         case (4)
            if(t_fact_flag(4).eqv..true.)then
-              hour=hour+t_fact*(i-1)
+              hour=hour_r+t_fact*(i-1)
            end if
         end select
 
@@ -580,7 +684,7 @@ subroutine read_file_grads( fname, val_name, dim_len, intvz, intvt, val )
         call read_file( trim(iname), nx, ny, read_flag+j,  &
   &                     val(:,:,j,i) )
      end do
-     write(*,*) "finishing to read file", trim(iname), nx, ny, read_flag+j, i
+     write(*,*) "finishing to read file", trim(iname), nx, ny, read_flag+intvz(1), read_flag+intvz(2), i
   end do
 
 end subroutine
