@@ -27,7 +27,7 @@ file = Array[
 dir = Array[ 
 "Alge_Solv",
 "Algebra",
-"Assimilation.f90",
+"Assimilation",
 "Basis",
 "Derivation",
 "ffts",
