@@ -1,3 +1,7 @@
+!-----------------------------------------------------------------------
+!     Copyright (C) 2000-2013 GFD Dennou Club. All rights reserved.
+!-----------------------------------------------------------------------
+
 module Trajectory
 !  ȥ饸ȥϤԤ롼
   use Statistics
@@ -722,7 +726,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -780,7 +784,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -823,7 +827,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -883,7 +887,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -926,7 +930,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -987,7 +991,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1030,7 +1034,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1073,7 +1077,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1133,7 +1137,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1176,7 +1180,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1219,7 +1223,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1263,7 +1267,7 @@ subroutine Stream_Line_2d( dt, step, ini_x, ini_y, x, y, u, v, trajx, trajy, opt
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.m==nx.or.n==ny)then
+        if(m<=0.or.n<=0.or.m==nx.or.n==ny)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1395,7 +1399,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1468,7 +1472,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1524,7 +1528,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1599,7 +1603,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1655,7 +1659,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1731,7 +1735,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1787,7 +1791,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1843,7 +1847,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1918,7 +1922,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -1974,7 +1978,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -2030,7 +2034,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
@@ -2086,7 +2090,7 @@ subroutine Stream_Line_3d( dt, step, ini_x, ini_y, ini_z, x, y, z, u, v, w, traj
 !-- , undef ƶϰ̲ᤷݤ.
 !-- ޤ, interpo_search ˤƸΰ賰ξ⤽ʹ߷׻Բǽ
 !-- ǤݤΤ, undef  undef ʤХȴ.
-        if(m==0.or.n==0.or.l==0.or.m==nx.or.n==ny.or.l==nz)then
+        if(m<=0.or.n<=0.or.l<=0.or.m==nx.or.n==ny.or.l==nz)then
            write(*,*) "*********** WARNING ********"
            write(*,*) "There is no parcel point in the calculating region."
            write(*,*) "Exit!!"
