@@ -1,3 +1,7 @@
+!---------------------------------------------------------------
+! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+!---------------------------------------------------------------
+
 module Algebra   ! 黻˹Ԥ⥸塼
 ! ͭͷ׻, 黻, ٥ȥ黻˹Ԥ.
 
@@ -31,14 +35,18 @@ subroutine rectangle_int( x, y, bot, top, res, undeff )  ! 1 ʬ
 !-- ʲʬ
 !-- ºݤˤ, ʬ˺Ƕ᤹
 
-  if(x(1)>bot)then
-     write(*,*) "#### WARNING ####"
-     write(*,*) "there is NOT the bot in the x(i)."  ! ΤȤ, i_bot=1ȤƤƤ, x(1)=bot ʤΤ, ;ʬ׻ϥȤʤƶϤʤ.
+  if(x(1)>=bot)then
+     if(x(1)>bot)then
+        write(*,*) "#### WARNING ####"
+        write(*,*) "there is NOT the bot in the x(i)."  ! ΤȤ, i_bot=1ȤƤƤ, x(1)=bot ʤΤ, ;ʬ׻ϥȤʤƶϤʤ.
+     end if
      i_bot=1
   end if
-  if(x(nx)<top)then
-     write(*,*) "#### WARNING ####"
-     write(*,*) "there is NOT the top in the x(i)."  ! ΤȤ, i_top=nx ȤƤ, x(nx)=top ʤΤ, ;ʬ׻ϥȤʤ.
+  if(x(nx)<=top)then
+     if(x(nx)<top)then
+        write(*,*) "#### WARNING ####"
+        write(*,*) "there is NOT the top in the x(i)."  ! ΤȤ, i_top=nx ȤƤ, x(nx)=top ʤΤ, ;ʬ׻ϥȤʤ.
+     end if
      i_top=nx
   end if
 
@@ -165,7 +173,7 @@ end subroutine abst
 
 !------------------------------------------
 
-subroutine radius(xp,yp,zp,x,y,z,rad)
+subroutine calc_radius(xp,yp,zp,x,y,z,rad)
   ! ֤εΥ׻롼
   ! Ĵ뤳Ȥˤ, 2 Ǥη׻ǽ.
   implicit none
@@ -193,7 +201,7 @@ subroutine radius(xp,yp,zp,x,y,z,rad)
 !$omp end parallel do
 
 
-end subroutine radius
+end subroutine calc_radius
 
 !------------------------------------------
 !------------------------------------------
