@@ -1,3 +1,7 @@
+!---------------------------------------------------------------
+! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+!---------------------------------------------------------------
+
 module Statistics  ! ײϴطΥ롼
 
   private :: summ
@@ -516,7 +520,7 @@ end subroutine covariance_3d
 !---------------------------------------
 !---------------------------------------
 
-subroutine nearest_search_1d( x, point, i )
+subroutine nearest_search_1d( x, point, i, hx, hp )
   ! 1 Ƕ˵õ롼
   ! interpo_search_1d ͤ, ͤ +1 ͤεΥӤ
   ! Υû򤹤.
@@ -524,6 +528,8 @@ subroutine nearest_search_1d( x, point, i )
   real, intent(in) :: x(:)  ! 
   real, intent(in) :: point  ! 
   integer, intent(inout) :: i  ! point κǶ˵ֹ
+  real, intent(in), optional :: hx(size(x))  ! x ɸΥ
+  real, intent(in), optional :: hp  ! point ǤΥ !! ޤѰդ
   real :: tmp1, tmp2
   integer :: j, nx
 
@@ -701,25 +707,37 @@ subroutine interpo_search_3d( x, y, z, pointx, pointy, pointz, i, j, k, undeff )
 end subroutine interpo_search_3d
 
 
-subroutine interpolation_1d( x, y, point, val )
+subroutine interpolation_1d( x, y, point, val, h, hp )
   ! 1 ޥ롼
   implicit none
   real, intent(in) :: x(2)  ! κü
   real, intent(in) :: y(2)  ! x Ƥ
   real, intent(in) :: point  ! 
   real, intent(inout) :: val  ! Ǥ
+  real, intent(in), optional :: h(2)  ! κǤΥ
+  real, intent(in), optional :: hp  ! ǤΥ
   real :: fd, dt
   real :: tmin
   real :: tmax
   real :: xmin
   real :: xmax
 
-  tmin=x(1)
-  tmax=x(2)
+  if(present(h))then
+     tmin=x(1)*h(1)
+     tmax=x(2)*h(2)
+  else
+     tmin=x(1)
+     tmax=x(2)
+  end if
+
   xmin=y(1)
   xmax=y(2)
 
-  dt=point-tmin
+  if(present(hp))then
+     dt=hp*point-tmin
+  else
+     dt=point-tmin
+  end if
 
   fd=(xmax-xmin)/(tmax-tmin)
 
@@ -727,7 +745,7 @@ subroutine interpolation_1d( x, y, point, val )
 end subroutine interpolation_1d
 
 
-subroutine interpolation_2d( x, y, z, point, val )
+subroutine interpolation_2d( x, y, z, point, val, h, hp )
   ! 2 νޥ롼
   ! ܥ롼ľľɸ֤ǤΤ߻Ѳǽ.
   implicit none
@@ -736,21 +754,36 @@ subroutine interpolation_2d( x, y, z, point, val )
   real, intent(in) :: z(2,2)  ! x, y ǤγǤ, (i,j) ˤĤ, i<=x, j<=y
   real, intent(in) :: point(2)  !  point(1)<=x ɸ, point(2)<=y ɸ
   real, intent(inout) :: val  ! Ǥ
+  real, intent(in), optional :: h(2,2)  ! λͶǤΥ
+  real, intent(in), optional :: hp(2)  ! ǤΥ
   real :: valx(2)
 
-  ! y(1) Ǥ x Ǥ
-  call interpolation_1d( (/x(1), x(2)/), (/z(1,1), z(2,1)/), point(1), valx(1) )
+  if(present(h))then
+     ! y(1) Ǥ x Ǥ
+     call interpolation_1d( x, (/z(1,1), z(2,1)/), point(1), valx(1),  &
+  &                         (/h(1,1), h(2,1)/), hp(1) )
 
-  ! y(2) Ǥ x Ǥ
-  call interpolation_1d( (/x(1), x(2)/), (/z(1,2), z(2,2)/), point(1), valx(2) )
+     ! y(2) Ǥ x Ǥ
+     call interpolation_1d( x, (/z(1,2), z(2,2)/), point(1), valx(2),  &
+  &                         (/h(1,2), h(2,2)/), hp(1) )
+   
+     ! x  y Ǥ(줬)
+     call interpolation_1d( y, valx, point(2), val, (/h(1,1), h(1,2)/), hp(2) )
+  else
+     ! y(1) Ǥ x Ǥ
+     call interpolation_1d( x, (/z(1,1), z(2,1)/), point(1), valx(1) )
 
-  ! x  y Ǥ(줬)
-  call interpolation_1d( (/y(1), y(2)/), (/valx(1), valx(2)/), point(2), val )
+     ! y(2) Ǥ x Ǥ
+     call interpolation_1d( x, (/z(1,2), z(2,2)/), point(1), valx(2) )
+   
+     ! x  y Ǥ(줬)
+     call interpolation_1d( y, valx, point(2), val )
+  end if
 
 end subroutine interpolation_2d
 
 
-subroutine interpolation_3d( x, y, z, u, point, val )
+subroutine interpolation_3d( x, y, z, u, point, val, h, hp )
   ! 3 νޥ롼
   ! ܥ롼ľľɸ֤ǤΤ߻Ѳǽ.
   implicit none
@@ -760,16 +793,31 @@ subroutine interpolation_3d( x, y, z, u, point, val )
   real, intent(in) :: u(2,2,2)  ! x, y, z ǤγǤ, (i,j,k) ˤĤ, i<=x, j<=y, k<=z
   real, intent(in) :: point(3)  !  point(1)<=x ɸ, point(2)<=y ɸ, point(3)<=z ɸ
   real, intent(inout) :: val  ! Ǥ
+  real, intent(in), optional :: h(2,2,2)  ! ȬǤΥ
+  real, intent(in), optional :: hp(3)  ! ǤΥ
   real :: valx(2)
 
-  ! z(1) Ǥ x-y ʿ̤Ǥνޤ
-  call interpolation_2d( x, y, u(:,:,1), point(1:2), valx(1) )
-
-  ! z(2) Ǥ x Ǥ
-  call interpolation_2d( x, y, u(:,:,2), point(1:2), valx(2) )
-
-  ! z(1)  z Ǥ(줬)
-  call interpolation_1d( (/z(1), z(2)/), (/valx(1), valx(2)/), point(3), val )
+  if(present(h))then
+     ! z(1) Ǥ x-y ʿ̤Ǥνޤ
+     call interpolation_2d( x, y, u(:,:,1), point(1:2), valx(1),  &
+  &                         h(:,:,1), hp(1:2) )
+   
+     ! z(2) Ǥ x Ǥ
+     call interpolation_2d( x, y, u(:,:,2), point(1:2), valx(2),  &
+  &                         h(:,:,2), hp(1:2) )
+   
+     ! z(1)  z Ǥ(줬)
+     call interpolation_1d( z, valx, point(3), val, h(1,1,:), hp(3) )
+  else
+     ! z(1) Ǥ x-y ʿ̤Ǥνޤ
+     call interpolation_2d( x, y, u(:,:,1), point(1:2), valx(1) )
+   
+     ! z(2) Ǥ x Ǥ
+     call interpolation_2d( x, y, u(:,:,2), point(1:2), valx(2) )
+   
+     ! z(1)  z Ǥ(줬)
+     call interpolation_1d( (/z(1), z(2)/), (/valx(1), valx(2)/), point(3), val )
+  end if
 
 end subroutine interpolation_3d
 
@@ -784,26 +832,26 @@ subroutine auto_interpolation_1d( x, r, u, v, undef )
   real, intent(in) :: r(:)  ! ޺ɸ
   real, intent(in) :: u(size(x))  ! ǡ
   real, intent(inout) :: v(size(r))  ! ޤǡ
-  integer, intent(in), optional :: undef  ! ̤
-  integer :: i, nx, nr, ix, ir
-  integer :: defun
-
-  if(present(undef))then
-     defun=undef
-  else
-     defun=0
-  end if
+  real, intent(in), optional :: undef  ! ̤
+  integer :: i, nx, nr, ir
+  real :: defun
 
   nx=size(x)
   nr=size(r)
 
-  do i=1, nr
-     call interpo_search_1d( x, r(i), ir, defun )
+  if(present(undef))then
+     defun=undef
+  else
+     defun=-2147483648
+  end if
 
-     if(ir/=defun.and.ir<nx)then
+  do i=1, nr
+     call interpo_search_1d( x, r(i), ir, int(defun) )
+
+     if(ir/=int(defun).and.ir<nx)then
         call interpolation_1d( x(ir:ir+1), u(ir:ir+1), r(i), v(i) )
      else
-        v(i)=real(defun)
+        v(i)=defun
      end if
   end do
 
@@ -822,30 +870,30 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef )
   real, intent(in) :: q(:)  ! ޺ɸ 2
   real, intent(in) :: u(size(x),size(y))  ! ǡ
   real, intent(inout) :: v(size(r),size(q))  ! ޤǡ
-  integer, intent(in), optional :: undef  ! ̤
-  integer :: i, j, nx, ny, nr, nq, ix, iy, ir, iq
-  integer :: defun
-
-  if(present(undef))then
-     defun=undef
-  else
-     defun=0
-  end if
+  real, intent(in), optional :: undef  ! ̤
+  integer :: i, j, nx, ny, nr, nq, ir, iq
+  real :: defun
 
   nx=size(x)
   ny=size(y)
   nr=size(r)
   nq=size(q)
 
+  if(present(undef))then
+     defun=undef
+  else
+     defun=-2147483648
+  end if
+
   do j=1, nq
      do i=1, nr
-        call interpo_search_2d( x, y, r(i), q(j), ir, iq, defun )
+        call interpo_search_2d( x, y, r(i), q(j), ir, iq, int(defun) )
 
-        if(ir/=defun.and.iq/=defun.and.ir<nx.and.iq<ny)then
+        if(ir/=int(defun).and.iq/=int(defun).and.ir<nx.and.iq<ny)then
            call interpolation_2d( x(ir:ir+1), y(iq:iq+1),  &
   &                               u(ir:ir+1,iq:iq+1), (/r(i), q(j)/), v(i,j) )
         else
-           v(i,j)=real(defun)
+           v(i,j)=defun
         end if
      end do
   end do
@@ -867,15 +915,9 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef )
   real, intent(in) :: p(:)  ! ޺ɸ 3
   real, intent(in) :: u(size(x),size(y),size(z))  ! ǡ
   real, intent(inout) :: v(size(r),size(q),size(p))  ! ޤǡ
-  integer, intent(in), optional :: undef  ! ̤
-  integer :: i, j, k, nx, ny, nz, nr, nq, np, ix, iy, iz, ir, iq, ip
-  integer :: defun
-
-  if(present(undef))then
-     defun=undef
-  else
-     defun=0
-  end if
+  real, intent(in), optional :: undef  ! ̤
+  integer :: i, j, k, nx, ny, nz, nr, nq, np, ir, iq, ip
+  real :: defun
 
   nx=size(x)
   ny=size(y)
@@ -884,19 +926,25 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef )
   nq=size(q)
   np=size(p)
 
+  if(present(undef))then
+     defun=undef
+  else
+     defun=-2147483648
+  end if
+
   do k=1, np
      do j=1, nq
         do i=1, nr
            call interpo_search_3d( x, y, z, r(i), q(j), p(k),  &
-  &                                ir, iq, ip, defun )
+  &                                ir, iq, ip, int(defun) )
 
-           if(ir/=defun.and.iq/=defun.and.ip/=defun.and.  &
+           if(ir/=int(defun).and.iq/=int(defun).and.ip/=int(defun).and.  &
   &           ir<nx.and.iq<ny.and.ip<nz)then
               call interpolation_3d( x(ir:ir+1), y(iq:iq+1), z(ip:ip+1),  &
   &                               u(ir:ir+1,iq:iq+1,ip:ip+1),  &
   &                               (/r(i), q(j), p(k)/), v(i,j,k) )
            else
-              v(i,j,k)=real(defun)
+              v(i,j,k)=defun
            end if
         end do
      end do
