@@ -1,3 +1,7 @@
+!-----------------------------------------------------------------------
+!     Copyright (C) 2000-2013 GFD Dennou Club. All rights reserved.
+!-----------------------------------------------------------------------
+
 module Thermo_Advanced_Routine
 ! å롼, ؿ use ʣǮϳشؿ׻⥸塼
 use Thermo_Function
@@ -18,7 +22,6 @@ subroutine Rich_horizon( za, pta, ptg, va, qva, qvs, Ri )
   real, intent(in), dimension(size(pta,1),size(pta,2)) :: qva  ! za Ǥκ [kg/kg]
   real, intent(in), dimension(size(pta,1),size(pta,2)) :: qvs  ! ɽ̤Ǥ˰º [kg/kg]
   real, intent(inout), dimension(size(pta,1),size(pta,2)) :: Ri  ! 㡼ɥ
-  real, dimension(size(pta,1),size(pta,2)) :: ptvg, ptva, dpt
   integer :: i, j, nx, ny
 
   nx=size(pta,1)
@@ -47,7 +50,6 @@ subroutine Louis_horizon( z, z0m, richard, Lo )
   real, intent(in), dimension(size(z0m,1),size(z0m,2)) :: richard  ! Х륯㡼ɥ
   real, intent(inout), dimension(size(z0m,1),size(z0m,2)) :: Lo  ! 
   real, parameter :: b=5.0, c=5.0
-  real :: cm_tmp, zratio
   integer :: i, j, nx, ny
 
   nx=size(z0m,1)
